; ===========================================================================
; ministorm
; ===========================================================================
;
; by losso/code red
; released at datastorm 2014
;
; NOTE: This is *NOT* the released party version, but rather
;       a what-if version with the "effect" I had in mind when
;       the deadline was approaching.
;
; ===========================================================================

	section code,code

	include "include/custom.i"

; ===========================================================================
; Memory layout
; ===========================================================================

SINTAB_SIZE		= $2000
WAVEFORM_SIZE	= $10000

sintab			= $1c000
sintab_end		= sintab+SINTAB_SIZE
waveform1		= sintab_end+WAVEFORM_SIZE
waveform2		= waveform1+WAVEFORM_SIZE
waveform3		= waveform2+WAVEFORM_SIZE
waveform4		= waveform3+WAVEFORM_SIZE
waveform4_end	= waveform4+WAVEFORM_SIZE

copper				= waveform4+WAVEFORM_SIZE
copper_end			= copper+128*4

screen				= copper_end+80*128
drawscreen			= screen+103*40
screen_end			= screen+80*2*256

chipsize			= screen_end-sintab

; ===========================================================================
; Init code for executables
; ===========================================================================

start:
	bra.b	bbstart

; ===========================================================================
; Bootblock start
; ===========================================================================

	dc.b	'DOS',0
	dc.l	0
	dc.b	' O< '

bbstart:
	lea		$dff000,a6

	move.w	#$7fff,d0
	move.w	d0,intena(a6)
	move.w	d0,intreq(a6)

	lea		copper,a1
	move.l	a1,cop1lc(a6)
	lea		copperlistdata(pc),a0
	move.w	#coppersize/4,d0
.copycop
	move.l	(a0)+,(a1)+
	dbf		d0,.copycop

	lea		drawscreen+20-4+40*60,a0
	lea		logo(pc),a1
	move	#5-1,d0
.copylogo
	move.l	(a1)+,(a0)+
	move.l	(a1)+,(a0)+
	lea		40-8(a0),a0
	dbf		d0,.copylogo

; ===========================================================================
; Calculate sine table
; ===========================================================================
; Only calculates the first quarter and mirrors the rest on the fly.
; Full period is $1000 words, $2000 bytes.
;
;     -$2000   0   $2000
; --------|----|-----|--------> y
;              |
; [$0000]      |:::..  <-- store sin(x) at   $0000++
; [$0800]      |:::''  <-- store  value at --$1000
; [$1000] ..:::|       <-- store -value at   $1000++
; [$1800] '':::|       <-- store -value at --$2000
; [$2000]      |
;              x
;
calcsine:
	lea		sintab,a0
	lea		$1000(a0),a1
	moveq	#0,d6
	move.w	#1024-1,d7
.calci
; ---------------------------------------------------------------------------
; Sine approximation (Bhaskara I)
; ---------------------------------------------------------------------------
	move.w	d6,d0
	sub.w	#$800,d0
	muls	d6,d0			; x*(x-$800)
	lsr.l	#8,d0
	move.w	d0,d1
 	muls	#-$1fffc/(1<<2),d0	; d0 = divisor
	add.w	#$500000/(1<<8),d1	; d1 = dividend
	divs	d1,d0
; ---------------------------------------------------------------------------
; Store and mirror other quarter periods
; ---------------------------------------------------------------------------
	move.w	d0,(a0)+		; +sin(x) at   $0000++
	move.w	d0,-(a1)		; +sin(x) at --$1000
	neg.w	d0
	move.w	d0,$1000-2(a0)		; -sin(x) at   $1000++
	move.w	d0,$1000(a1)		; -sin(x) at --$2000
	addq	#1,d6
	dbf		d7,.calci
calcsine_end:

; ===========================================================================
; Render instruments
; ===========================================================================
render:
	lea		vars(pc),a6
	lea		waveform1,a0
	lea		note_steps(pc),a2

	lea		voice_first(pc),a1
	lea		osc_sin(pc),a3
	lea		sintab,a5
	bsr		render_voice

	move.l	#(2000<<16)|2,INSTR_LEN(a6)
	lea		WAVEFORM_SIZE(a0),a0
	;lea	voice_bass(pc),a1
	lea		osc_bass(pc),a3
	bsr		render_voice

	lea		WAVEFORM_SIZE/2(a0),a0
	lea		-voice_bass_len(a1),a1
	bsr		render_voice

	move.l	#(6000<<16)|3,INSTR_LEN(a6)
	lea		WAVEFORM_SIZE/2(a0),a0
	lea		osc_sin(pc),a3
	;lea	voice_second(pc),a1
	bsr		render_voice

	lea		WAVEFORM_SIZE(a0),a0
	moveq	#16-1,d7
.basss
	move.l	#NOTE_C5_VALUE<<10,d1
	move.l	a0,a4
	bsr		osc_bassdrum
	lea		$10000/16(a0),a0
	dbf		d7,.basss

render_end:
	move.w	#417,d0
	moveq	#4-1,d1
	lea		$dff000+aud0,a6
	move.l	#waveform1,d2
.initloop
	move.l	d2,(a6)+
	move.w	#WAVEFORM_SIZE/2-1,(a6)+
	move.w	d0,(a6)+
	move.w	#0,(a6)+
	lea		ac_SIZEOF-4-2-2-2(a6),a6

	add.l	#WAVEFORM_SIZE,d2
	dbf		d1,.initloop

	;		  +--------------- SET/CLR
	;		  |     +--------- DMA
	;		  |     |+-------- BPL
	;		  |     ||+------- COPPER
	;		  |     |||   3210 AUDIO
	move.w	#%1000001110001111,dmacon-aud0-ac_SIZEOF*4(a6)

	bset	#1,$bfe001

	move.w	#0,copper+bgcol_off
	move.w	#0,copper+logocol_off

; ===========================================================================
; Main loop
; ===========================================================================
PATTERN_LENGH_TICKS = 550
BALL_CENTER = 38
	moveq	#0,d7
	lea		drawscreen,a0
	lea		sintab+$1000,a1
	moveq	#BALL_CENTER,d1
	moveq	#0,d3

hui:
	lea		vars+INSTR_VOLS(pc),a5
	lea		$dff000,a6
waitvbr:
	btst	#0,vposr+1(a6)
	beq		waitvbr
waitvbr2:
	btst	#0,vposr+1(a6)
	bne		waitvbr2

; ---------------------------------------------------------------------------
; Subsequently fade in channels
; ---------------------------------------------------------------------------
	moveq	#0,d4
	moveq	#4-1,d2
.checkloop
	cmp.w	d4,d7
	blt		.nofadein
	add.w	#1,(a5)
.nofadein
	addq	#2,a5
	add.w	#PATTERN_LENGH_TICKS,d4
	dbf		d2,.checkloop

; logo fade-in
 move.w	-2(a5),d6
 lsr	#2+5,d6		; $00..$0f
 btst	#4,d6
 bne	.nologofade
 move.w	d6,d4
 lsl	#4,d4
 add.w	d4,d6
 lsl	#4,d4
 add.w	d4,d6
 move.w	d6,copper+logocol_off
.nologofade

; Main voice solo?
	cmp.w	#2954,d7
	bne		.notnow
	moveq	#1,d3
.notnow
	bcs		.novoice
	move.l	#0,-2*4(a5)
	move.l	#64*32,2*2-2*4(a5)
	move.w	d0,copper+bgcol_off
.novoice

PATLEN = 200

; ---------------------------------------------------------------------------
; Set volumes
; ---------------------------------------------------------------------------
	moveq	#4-1,d2
	lea		aud0(a6),a4
.volset
	move.w	-(a5),d4
	cmp.w	#64*32,d4
	bhi		.novol
	lsr.w	#5,d4
	move.w	d4,ac_vol(a4)
.novol
	lea		ac_SIZEOF(a4),a4
	dbf		d2,.volset

	move.w	#60*40/4-1,d6
	move.l	a0,a3
.clrball
	move.l	#0,(a3)+
	dbf		d6,.clrball

BEAT_TICKS = 24

 cmp.w	#BALL_CENTER+24,d1
 blt	.noturn
 moveq	#-1,d3
 bra	.noturn2
.noturn
 cmp.w	#BALL_CENTER-24,d1
 bgt	.noturn2
 moveq	#1,d3
.noturn2
 add.w	d3,d1

	move.l	d7,d6
	divu	#BEAT_TICKS,d6
	swap	d6					; ticks % BEAT_TICKS
	mulu	#1024/6,d6
	and.w	#$ffe,d6
	move.w	(a1,d6.w),d6		; $0000..$2000
	asr.w	#8,d6				; $0000..$0020

	move.w	d6,d0	; copy for bg col
	asr		#2,d0
	bpl		.nonot
	neg		d0
.nonot
	and		#$f,d0

	add.w	#50,d6
	mulu	#40,d6
	moveq	#8-1,d5

	move.w	d1,d4				; x pos
	lsr.w	#4,d4
	add.w	d4,d4
	add.w	d4,d6
	move.w	d1,d4
	and.w	#$f,d4
	move.l	#$ff0000,d2
	lsr.l	d4,d2

.ball
	move.l	d2,18-4(a0,d6.w)
	add.w	#40,d6
	dbf		d5,.ball
	move.w	#0,(a0)

	addq	#1,d7
	bra.b	hui

vars:
	dc.w	2100		; instr_len
	dc.w	0			; instr_volshift
	dc.w	0,0,0,0		; volumes

INSTR_LEN = 0
INSTR_VOLSHIFT = 2
INSTR_VOLS = 4

	include "rendervoice.asm"

	even

copperlistdata:
bgcol:
	dc.w	$0180,$0fff
	dc.w	$0182,$0fff
	dc.w	$00e0,(screen&$ffff0000)>>16
	dc.w	$00e2,(screen&$ffff)
	dc.l	$008e2cd1,$00902c41
	dc.l	$00920038,$009400d0
	;dc.w	$0108,0
	dc.w	$9001,$fffe
	dc.w	$0100,$1200

	dc.w	$9001+60*$100,$fffe
logocol:
	dc.w	$0182,$0fff

	dc.w	$9601+60*$100,$fffe
	dc.w	$ffff,$fffe
coppersize = *-copperlistdata
bgcol_off   = bgcol-copperlistdata+2
logocol_off = logocol-copperlistdata+2

logo:
	dc.l	%000011011111101100001111110,%11111011000011111101111101111111
	dc.l	%111111000001101111100000110,%11000011111011001101101101101011
	dc.l	%110011011111101100001111110,%11111011000011001101100001101011
	dc.l	%110011011001101100001100110,%00011011000011001101100001100011
	dc.l	%111111011111101111101111110,%11111011111011111101100001100011


; ===========================================================================
; Music data 
; ===========================================================================

INSTR_VOICE		= 1
INSTR_BASSDRUM	= 2
INSTR_BASS		= 3
INSTR_HIHAT		= 4

BASE_T = 1
BASE_STEP = 3

NOTE_C5_VALUE = BASE_STEP*BASE_T*2616
NOTE_D5_VALUE = BASE_STEP*BASE_T*2937
NOTE_E5_VALUE = BASE_STEP*BASE_T*3296
NOTE_F5_VALUE = BASE_STEP*BASE_T*3492
NOTE_G5_VALUE = BASE_STEP*BASE_T*3920
NOTE_C6_VALUE = NOTE_C5_VALUE*2;BASE_STEP*BASE_T*5544
NOTE_D6_VALUE = BASE_STEP*BASE_T*5873
NOTE_E6_VALUE = BASE_STEP*BASE_T*6592
NOTE_F6_VALUE = BASE_STEP*BASE_T*6985
NOTE_G6_VALUE = BASE_STEP*BASE_T*7840

NOTE_C5 = 0*2
NOTE_D5 = 1*2
NOTE_E5 = 2*2
NOTE_F5 = 3*2
NOTE_G5 = 4*2
NOTE_C6 = 5*2
NOTE_D6 = 6*2
NOTE_E6 = 7*2
NOTE_F6 = 8*2
NOTE_G6 = 9*2

ROWLEN = 1

musicdata:

note_steps:
	dc.w	NOTE_C5_VALUE
	dc.w	NOTE_D5_VALUE
	dc.w	NOTE_E5_VALUE
	dc.w	NOTE_F5_VALUE
	dc.w	NOTE_G5_VALUE
	dc.w	NOTE_C6_VALUE
	dc.w	NOTE_D6_VALUE
	dc.w	NOTE_E6_VALUE
	dc.w	NOTE_F6_VALUE
	dc.w	NOTE_G6_VALUE

voice_first:
	dc.b	60*ROWLEN,	NOTE_G6	; sail
	dc.b	62*ROWLEN,	NOTE_D6	; a-
	dc.b	 0*ROWLEN,	NOTE_F6	; way,

	dc.b	 4*ROWLEN,	NOTE_F6	; sail
	dc.b	 6*ROWLEN,	NOTE_C6	; a-
	dc.b	 8*ROWLEN,	NOTE_E6	; way,

	dc.b	12*ROWLEN,	NOTE_E6	; sail
	dc.b	14*ROWLEN,	NOTE_C6	; a-
	dc.b	16*ROWLEN,	NOTE_D6	; way!

	dc.b	28*ROWLEN,	NOTE_G5	; sail ; REPEAT one octave lower at 32 (modulo 64)
	dc.b	30*ROWLEN,	NOTE_D5	; a-
	dc.b	32*ROWLEN,	NOTE_F5	; way,

	dc.b	36*ROWLEN,	NOTE_F5	; sail
	dc.b	38*ROWLEN,	NOTE_C5	; a-
	dc.b	40*ROWLEN,	NOTE_E5	; way,

	dc.b	44*ROWLEN,	NOTE_E5	; sail
	dc.b	46*ROWLEN,	NOTE_C5	; a-
	dc.b	48*ROWLEN,	NOTE_D5	; way!
	dc.b	-1
voice_bass:
	dc.b	 0*ROWLEN,	NOTE_F5
	dc.b	 2*ROWLEN,	NOTE_F6
	dc.b	 6*ROWLEN,	NOTE_C5
	dc.b	 8*ROWLEN,	NOTE_C6
	dc.b	14*ROWLEN,	NOTE_D6
	dc.b	16*ROWLEN,	NOTE_G5
	dc.b	18*ROWLEN,	NOTE_G6
	dc.b	22*ROWLEN,	NOTE_G5
	dc.b	24*ROWLEN,	NOTE_G6
	dc.b	-1
voice_bass_end:
voice_bass_len = voice_bass_end-voice_bass
voice_second:
	dc.b	 0*ROWLEN,	NOTE_F5
	dc.b	 8*ROWLEN,	NOTE_C5
	dc.b	16*ROWLEN,	NOTE_G5
	dc.b	 (0+32)*ROWLEN,	NOTE_F5	; REPEAT at 32
	dc.b	 (8+32)*ROWLEN,	NOTE_C5
	dc.b	(16+32)*ROWLEN,	NOTE_G5
	dc.b	-1

musicdata_end:

	dc.b	'15 bytes left:)'